package javax.swing.plaf.basic;

import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.awt.event.MouseEvent;
import javax.swing.ButtonModel;
import javax.swing.JRootPane;
import javax.swing.JButton;
import java.awt.event.FocusEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.KeyStroke;
import javax.swing.plaf.ComponentInputMapUIResource;
import sun.swing.DefaultLookup;
import javax.swing.InputMap;
import javax.swing.SwingUtilities;
import javax.swing.JComponent;
import java.beans.PropertyChangeEvent;
import javax.swing.AbstractButton;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeListener;
import java.awt.event.FocusListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;

public class BasicButtonListener implements MouseListener, MouseMotionListener, FocusListener, ChangeListener, PropertyChangeListener
{
    private long lastPressedTimestamp;
    private boolean shouldDiscardRelease;
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicButtonListener.Actions("pressed"));
        lazyActionMap.put(new BasicButtonListener.Actions("released"));
    }
    
    public BasicButtonListener(AbstractButton abstractButton) {
        this.lastPressedTimestamp = -1L;
        this.shouldDiscardRelease = false;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        String propertyName = propertyChangeEvent.getPropertyName();
        if (propertyName == "mnemonic") {
            this.updateMnemonicBinding((AbstractButton)propertyChangeEvent.getSource());
        }
        else if (propertyName == "contentAreaFilled") {
            this.checkOpacity((AbstractButton)propertyChangeEvent.getSource());
        }
        else if (propertyName == "text" || "font" == propertyName || "foreground" == propertyName) {
            AbstractButton abstractButton = (AbstractButton)propertyChangeEvent.getSource();
            BasicHTML.updateRenderer(abstractButton, abstractButton.getText());
        }
    }
    
    protected void checkOpacity(AbstractButton abstractButton) {
        abstractButton.setOpaque(abstractButton.isContentAreaFilled());
    }
    
    public void installKeyboardActions(JComponent component) {
        this.updateMnemonicBinding((AbstractButton)component);
        LazyActionMap.installLazyActionMap(component, BasicButtonListener.class, "Button.actionMap");
        SwingUtilities.replaceUIInputMap(component, 0, this.getInputMap(0, component));
    }
    
    public void uninstallKeyboardActions(JComponent component) {
        SwingUtilities.replaceUIInputMap(component, 2, null);
        SwingUtilities.replaceUIInputMap(component, 0, null);
        SwingUtilities.replaceUIActionMap(component, null);
    }
    
    InputMap getInputMap(int n, JComponent c) {
        if (n == 0) {
            BasicButtonUI ui = (BasicButtonUI)BasicLookAndFeel.getUIOfType(((AbstractButton)c).getUI(), BasicButtonUI.class);
            if (ui != null) {
                return (InputMap)DefaultLookup.get(c, ui, ui.getPropertyPrefix() + "focusInputMap");
            }
        }
        return null;
    }
    
    void updateMnemonicBinding(AbstractButton abstractButton) {
        int mnemonic = abstractButton.getMnemonic();
        if (mnemonic != 0) {
            InputMap uiInputMap2 = SwingUtilities.getUIInputMap(abstractButton, 2);
            if (uiInputMap2 == null) {
                uiInputMap2 = new ComponentInputMapUIResource(abstractButton);
                SwingUtilities.replaceUIInputMap(abstractButton, 2, uiInputMap2);
            }
            uiInputMap2.clear();
            uiInputMap2.put(KeyStroke.getKeyStroke(mnemonic, BasicLookAndFeel.getFocusAcceleratorKeyMask(), false), "pressed");
            uiInputMap2.put(KeyStroke.getKeyStroke(mnemonic, BasicLookAndFeel.getFocusAcceleratorKeyMask(), true), "released");
            uiInputMap2.put(KeyStroke.getKeyStroke(mnemonic, 0, true), "released");
        }
        else {
            InputMap uiInputMap = SwingUtilities.getUIInputMap(abstractButton, 2);
            if (uiInputMap != null) {
                uiInputMap.clear();
            }
        }
    }
    
    @Override
    public void stateChanged(ChangeEvent changeEvent) {
        ((AbstractButton)changeEvent.getSource()).repaint();
    }
    
    @Override
    public void focusGained(FocusEvent focusEvent) {
        AbstractButton abstractButton = (AbstractButton)focusEvent.getSource();
        if (abstractButton instanceof JButton && ((JButton)abstractButton).isDefaultCapable()) {
            JRootPane rootPane = abstractButton.getRootPane();
            if (rootPane != null) {
                BasicButtonUI ui = (BasicButtonUI)BasicLookAndFeel.getUIOfType(abstractButton.getUI(), BasicButtonUI.class);
                if (ui != null && DefaultLookup.getBoolean(abstractButton, ui, ui.getPropertyPrefix() + "defaultButtonFollowsFocus", true)) {
                    rootPane.putClientProperty("temporaryDefaultButton", abstractButton);
                    rootPane.setDefaultButton((JButton)abstractButton);
                    rootPane.putClientProperty("temporaryDefaultButton", null);
                }
            }
        }
        abstractButton.repaint();
    }
    
    @Override
    public void focusLost(FocusEvent focusEvent) {
        AbstractButton c = (AbstractButton)focusEvent.getSource();
        JRootPane rootPane = c.getRootPane();
        if (rootPane != null) {
            JButton defaultButton = (JButton)rootPane.getClientProperty("initialDefaultButton");
            if (c != defaultButton) {
                BasicButtonUI ui = (BasicButtonUI)BasicLookAndFeel.getUIOfType(c.getUI(), BasicButtonUI.class);
                if (ui != null && DefaultLookup.getBoolean(c, ui, ui.getPropertyPrefix() + "defaultButtonFollowsFocus", true)) {
                    rootPane.setDefaultButton(defaultButton);
                }
            }
        }
        ButtonModel model = c.getModel();
        model.setPressed(false);
        model.setArmed(false);
        c.repaint();
    }
    
    @Override
    public void mouseMoved(MouseEvent mouseEvent) {
    }
    
    @Override
    public void mouseDragged(MouseEvent mouseEvent) {
    }
    
    @Override
    public void mouseClicked(MouseEvent mouseEvent) {
    }
    
    @Override
    public void mousePressed(MouseEvent anEvent) {
        if (SwingUtilities.isLeftMouseButton(anEvent)) {
            AbstractButton abstractButton = (AbstractButton)anEvent.getSource();
            if (abstractButton.contains(anEvent.getX(), anEvent.getY())) {
                long multiClickThreshhold = abstractButton.getMultiClickThreshhold();
                long lastPressedTimestamp = this.lastPressedTimestamp;
                long when = anEvent.getWhen();
                this.lastPressedTimestamp = when;
                long n = when;
                if (lastPressedTimestamp != -1L && n - lastPressedTimestamp < multiClickThreshhold) {
                    this.shouldDiscardRelease = true;
                    return;
                }
                ButtonModel model = abstractButton.getModel();
                if (!model.isEnabled()) {
                    return;
                }
                if (!model.isArmed()) {
                    model.setArmed(true);
                }
                model.setPressed(true);
                if (!abstractButton.hasFocus() && abstractButton.isRequestFocusEnabled()) {
                    abstractButton.requestFocus();
                }
            }
        }
    }
    
    @Override
    public void mouseReleased(MouseEvent anEvent) {
        if (SwingUtilities.isLeftMouseButton(anEvent)) {
            if (this.shouldDiscardRelease) {
                this.shouldDiscardRelease = false;
                return;
            }
            ButtonModel model = ((AbstractButton)anEvent.getSource()).getModel();
            model.setPressed(false);
            model.setArmed(false);
        }
    }
    
    @Override
    public void mouseEntered(MouseEvent anEvent) {
        AbstractButton abstractButton = (AbstractButton)anEvent.getSource();
        ButtonModel model = abstractButton.getModel();
        if (abstractButton.isRolloverEnabled() && !SwingUtilities.isLeftMouseButton(anEvent)) {
            model.setRollover(true);
        }
        if (model.isPressed()) {
            model.setArmed(true);
        }
    }
    
    @Override
    public void mouseExited(MouseEvent mouseEvent) {
        AbstractButton abstractButton = (AbstractButton)mouseEvent.getSource();
        ButtonModel model = abstractButton.getModel();
        if (abstractButton.isRolloverEnabled()) {
            model.setRollover(false);
        }
        model.setArmed(false);
    }
    
    private static class Actions extends UIAction
    {
        private static final String PRESS = "pressed";
        private static final String RELEASE = "released";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            AbstractButton abstractButton = (AbstractButton)actionEvent.getSource();
            String name = this.getName();
            if (name == "pressed") {
                ButtonModel model2 = abstractButton.getModel();
                model2.setArmed(true);
                model2.setPressed(true);
                if (!abstractButton.hasFocus()) {
                    abstractButton.requestFocus();
                }
            }
            else if (name == "released") {
                ButtonModel model = abstractButton.getModel();
                model.setPressed(false);
                model.setArmed(false);
            }
        }
        
        public boolean isEnabled(Object o) {
            return o == null || !(o instanceof AbstractButton) || ((AbstractButton)o).getModel().isEnabled();
        }
    }
}
